# Advanced fastlane

### Environment Variables
You can define environment variables in a `.env` or `.env.default` file in the same directory as your `Fastfile`. Environment variables are loading using [dotenv](https://github.com/bkeepers/dotenv). Here's an example.

```
WORKSPACE=YourApp.xcworkspace
HOCKEYAPP_API_TOKEN=your-hockey-api-token
```

`fastlane` also has a `--env` option that allows loading of environment specific `dotenv` files. `.env` and `.env.default` will be loaded before environment specific `dotenv` files are loaded. The naming convention for environment specific `dotenv` files is `.env.<environment>`

For example, `fastlane <lane-name> --env development` will load `.env`, `.env.default`, and `.env.development`

## Lane Context

The different actions can *communicate* with each other using a shared hash. You can access them in your lanes with the following code.

```ruby
Actions.lane_context[VARIABLE_NAME_HERE] # the name of the current lane
```

Replace `VARIABLE_NAME_HERE` with any of the following.

```ruby
Actions::SharedValues::LANE_NAME                 # The name of the current lane
Actions::SharedValues::BUILD_NUMBER              # Generated by `increment_build_number`
Actions::SharedValues::SNAPSHOT_SCREENSHOTS_PATH # Generated by `snapshot`
Actions::SharedValues::PRODUCE_APPLE_ID          # The Apple ID of the newly created app
Actions::SharedValues::IPA_OUTPUT_PATH           # Generated by `ipa`
Actions::SharedValues::DSYM_OUTPUT_PATH          # Generated by `ipa`
Actions::SharedValues::SIGH_PROFILE_PATH         # Generated by `sigh`
Actions::SharedValues::SIGH_UDID                 # The UDID of the generated provisioning profile
Actions::SharedValues::HOCKEY_DOWNLOAD_LINK      # Generated by `hockey`
Actions::SharedValues::DEPLOYGATE_URL            # Generated by `deploygate`
Actions::SharedValues::DEPLOYGATE_APP_REVISION   # Integer, generated by `deploygate`
Actions::SharedValues::DEPLOYGATE_APP_INFO       # Hash, generated by `deploygate`
````

Keep in mind the ```before_all``` and ```after_all``` block will be executed for _each_ of the ```lanes```.

## Hide the `fastlane` folder

Just rename the folder to `.fastlane` in case you don't want it to be visible in the Finder.

## Load own actions from external folder

Add this to the top of your `Fastfile`.

```ruby
actions_path '../custom_actions_folder/'
```

## The Appfile

The `Appfile` stores useful information that are used across all `fastlane` tools like your *Apple ID* or the application *Bundle Identifier*, to deploy your lanes faster and tailored on your project needs. 

By default an Appfile looks like:

```ruby
app_identifier "net.sunapps.1" # The bundle identifier of your app
apple_id "felix@krausefx.com" # Your Apple email address

# You can uncomment the lines below and add your own 
# team selection in case you're in multiple teams
# team_name "Felix Krause"
# team_id "Q2CBPJ58CA"
```

If your project has different bundle identifiers per environment (i.e. beta, app store), you can define that by using `for_platform` and/or `for_lane` block declaration. 

```ruby
app_identifier "net.sunapps.1"
apple_id "felix@krausefx.com"
team_id "Q2CBPJ58CC"

for_lane "ios beta" do
  app_identifier "net.sunapps.1.beta"
end

for_lane "ios enterprise" do
  app_identifier "enterprise.com"
end

for_platform :ios do
  team_id '123' # for all iOS related things
  for_lane :test do
    app_identifier 'com.app.test'
  end
end
```

You only have to prefix with `ios` if you're using `platform :ios do` in your `Fastfile`.

`fastlane` will always use the lane specific value if given, otherwise fall back to the value on the top of the file. Therefore, while driving the `:beta` lane, this configuration is loaded:

```ruby
app_identifier "net.sunapps.1.beta"
apple_id "felix@krausefx.com"
team_id "Q2CBPJ58CC"
```

## Skip update check when launching `fastlane`

You can set the environment variable `FASTLANE_SKIP_UPDATE_CHECK` to skip the update check.
